/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
import React from "react";
import Moment from "react-moment";
import TaskDescription from "../TaskDescription/TaskDescription";
import { DataTableColumn } from "@kie-tools/runtime-tools-components/dist/components/DataTable";
import { UserTaskInstance } from "@kie-tools/runtime-tools-process-gateway-api/dist/types";
import { TaskState } from "../../../../taskDetails";

export const getDefaultColumn = (columnPath: string, columnLabel: string, isSortable: boolean): DataTableColumn => {
  return {
    path: columnPath,
    label: columnLabel,
    isSortable,
  };
};

export const getDateColumn = (columnPath: string, columnLabel: string): DataTableColumn => {
  return {
    label: columnLabel,
    path: columnPath,
    bodyCellTransformer: (value) => <Moment fromNow>{new Date(`${value}`)}</Moment>,
    isSortable: true,
  };
};

export const getTaskDescriptionColumn = (selectTask: (task: UserTaskInstance) => void): DataTableColumn => {
  return {
    label: "Name",
    path: "referenceName",
    bodyCellTransformer: (cellValue, rowTask) => {
      return (
        <TaskDescription task={rowTask as UserTaskInstance} onClick={() => selectTask(rowTask as UserTaskInstance)} />
      );
    },
    isSortable: true,
  };
};

export const getTaskStateColumn = (): DataTableColumn => {
  return {
    label: "Status",
    path: "state",
    bodyCellTransformer: (cellValue, rowTask) => <TaskState task={rowTask as UserTaskInstance} />,
    isSortable: true,
  };
};

export const getDefaultTaskStates = (): string[] => {
  return ["Ready", "Reserved", "Completed", "Aborted", "Skipped"];
};

export const getDefaultActiveTaskStates = (): string[] => {
  return ["Ready", "Reserved"];
};
