/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
module.exports = ConfirmTravelFormDraft7 = {
  $schema: "http://json-schema.org/draft-07/schema#",
  type: "object",
  definitions: {
    Address: {
      type: "object",
      properties: {
        street: {
          type: "string",
        },
        city: {
          type: "string",
        },
        zipCode: {
          type: "string",
        },
        country: {
          type: "string",
        },
      },
    },
    Flight: {
      type: "object",
      properties: {
        flightNumber: {
          type: "string",
        },
        seat: {
          type: "string",
        },
        gate: {
          type: "string",
        },
        departure: {
          type: "string",
          format: "date-time",
        },
        arrival: {
          type: "string",
          format: "date-time",
        },
      },
    },
    Hotel: {
      type: "object",
      properties: {
        name: {
          type: "string",
        },
        address: {
          $ref: "#/definitions/Address",
        },
        phone: {
          type: "string",
        },
        bookingNumber: {
          type: "string",
        },
        room: {
          type: "string",
        },
      },
    },
  },
  properties: {
    flight: {
      allOf: [{ $ref: "#/definitions/Flight" }, { input: true }],
    },
    hotel: {
      allOf: [{ $ref: "#/definitions/Hotel" }, { input: true }, { output: true }],
    },
  },
  phases: ["complete", "release"],
};
